import os
import re

from termcolor import colored


def validate_projects_folder_path(projects_folder_path):
    msg = None

    if not projects_folder_path:
        msg = colored(
            ("\n\nERROR: You must provide the path to your projects folder."),
            "red",
            attrs=["bold"],
        )
    elif not os.path.exists(projects_folder_path):
        msg = colored(
            (
                f"\n\nERROR: {projects_folder_path} does not exist. "
                "Please provide a valid path."
            ),
            "red",
            attrs=["bold"],
        )

    if msg:
        print(msg)
        exit(1)


def validate_project_name(project_path):
    project_name = os.path.basename(project_path)
    msg = None

    if not project_name:
        msg = colored(
            ("\n\nERROR: You must provide the name of the project."),
            "red",
            attrs=["bold"],
        )

    elif os.path.exists(project_path):
        msg = colored(
            (
                f"\n\nERROR: {project_path} already exists. "
                "Please choose another name or delete the existing folder."
            ),
            "red",
            attrs=["bold"],
        )
    elif not re.match(r"^[a-zA-Z0-9-_]+$", project_name):
        msg = colored(
            (
                "\n\nERROR: Invalid project name. "
                "Project name can only contain letters, numbers, dashes, and underscores."
            ),
            "red",
            attrs=["bold"],
        )

    if msg:
        print(msg)
        exit(1)



import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
SCRIPTS_DIR = os.path.join(BASE_DIR, "ppieces/scripts")
TEMPLATES_DIR = os.path.join(BASE_DIR, "ppieces/templates")



import os
import shutil

from termcolor import colored

from ppieces.utils.constants import TEMPLATES_DIR


def copy_template_file(filename, project_path):
    shutil.copy(
        os.path.join(TEMPLATES_DIR, filename),
        os.path.join(project_path, filename),
    )
    msg = colored(
        (f"Created a default {filename} file in {project_path}"),
        "yellow",
        attrs=["bold"],
    )
    print(msg)


def copy_requirements_file(project_path):
    copy_template_file("requirements.txt", project_path)


def copy_makefile(project_path):
    copy_template_file("Makefile", project_path)


def copy_main_file(project_path):
    copy_template_file("main.py", project_path)


def copy_precommit_config(project_path):
    copy_template_file(".pre-commit-config.yaml", project_path)


def copy_ruff_config(project_path):
    copy_template_file(".ruff.toml", project_path)


def copy_gitignore_file(project_path):
    copy_template_file(".gitignore", project_path)


def copy_readme_file(project_path, username):
    copy_template_file("README-sample.md", project_path)
    project_name = os.path.basename(project_path)
    with open(os.path.join(project_path, "README-sample.md"), "r") as f:
        readme = f.read()
        readme = readme.replace("{{project_name}}", project_name)
        readme = readme.replace("{{username}}", username)
    with open(os.path.join(project_path, "README-sample.md"), "w") as f:
        f.write(readme)
    os.rename(
        os.path.join(project_path, "README-sample.md"),
        os.path.join(project_path, "README.md"),
    )



import os

from termcolor import colored

from ppieces.utils.commands import (
    create_project_directory,
    delete_path,
)
from ppieces.utils.flows import finalize_project, get_project_path, setup_project
from ppieces.utils.prompts import ask_user, bye, welcome


def run_cli(
    non_interactive,
    project_folder,
    project_name,
    virtual_env,
    git,
    pre_commit,
    ruff,
    autoenv,
    makefile,
    username,
):
    project_path = None
    try:
        if non_interactive:
            project_path = os.path.join(project_folder, project_name)
            create_project_directory(project_path, interactive=False)

            options = {
                "virtual_env": virtual_env,
                "git": git,
                "autoenv": autoenv,
                "ruff": ruff,
                "pre_commit": pre_commit,
                "makefile": makefile,
            }

        else:
            welcome()
            project_path = get_project_path()
            project_name = os.path.basename(project_path)
            create_project_directory(project_path, interactive=True)

            options = {
                "virtual_env": ask_user(
                    f"Do you want to create a virtual environment for `{project_name}` "
                    "project?"
                ),
                "git": ask_user(
                    f"Do you want to initialize a git repository for `{project_name}` "
                    "project?"
                ),
                "autoenv": ask_user(
                    f"Do you want to autoenv config files for `{project_name}` project?"
                ),
                "ruff": ask_user(
                    f"Do you want to add a ruff config file for `{project_name}` project?"
                ),
                "pre_commit": ask_user(
                    f"Do you want to add a pre-commit config file for `{project_name}` "
                    "project?"
                ),
                "makefile": ask_user(
                    f"Do you want to add a Makefile for `{project_name}` project?"
                ),
            }
            git = options["git"]

        setup_project(project_path, options, username)
        finalize_project(project_path, git, pre_commit)
        bye()

    except KeyboardInterrupt:
        print(colored("\nAborting...", "red", attrs=["bold"]))
        if project_path and os.path.exists(project_path):
            delete_path(project_path)



import os
import shutil
import subprocess
import sys

import click

from rich.console import Console
from termcolor import colored

from ppieces.utils.constants import SCRIPTS_DIR
from ppieces.utils.copy import (
    copy_gitignore_file,
    copy_readme_file,
    copy_requirements_file,
)
from ppieces.utils.prompts import ask_user

console = Console()


def check_precommit(git):
    if not git:
        msg = colored(
            (
                "Cannot install pre-commit without git. "
                "Config file copied but not installing hooks..."
            ),
            "red",
            attrs=["bold"],
        )
        print(msg)
        return False

    try:
        with console.status("[green]Checking if pre-commit is installed..."):
            subprocess.run(
                ["pre-commit", "--version"],
                check=True,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
    except Exception:
        msg = colored(
            (
                "pre-commit is not installed. Please make sure to install it\n"
                "pipx install pre-commit or brew install pre-commit"
            ),
            "red",
            attrs=["bold"],
        )
        print(msg)

        msg = colored(
            ("Continuing without pre-commit. You can install it later.\n"),
            "blue",
            attrs=["bold"],
        )
        print(msg)
        return False

    return True


def install_precommit_hooks(project_path):
    with console.status("[green]Installing pre-commit hooks..."):
        current_dir = os.getcwd()
        os.chdir(project_path)
        subprocess.run(
            ["pre-commit", "install"],
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )
        os.chdir(current_dir)

    msg = colored(
        ("Installed pre-commit hooks successfully!"),
        "yellow",
        attrs=["bold"],
    )
    print(msg)


def setup_autoenv(project_path):
    subprocess.run(
        [os.path.join(f"{SCRIPTS_DIR}", "setup_autoenv.sh"), project_path],
        check=True,
    )
    msg = colored(
        ("autoenv configured successfully!"),
        "yellow",
        attrs=["bold"],
    )
    print(msg)


def add_and_install_requirements(project_path):
    copy_requirements_file(project_path)

    with console.status("[green]Installing default requirements..."):
        subprocess.run(
            [
                f"{project_path}/venv/bin/pip",
                "install",
                "-r",
                os.path.join(project_path, "requirements.txt"),
                "--upgrade",
            ],
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )
    msg = colored(
        ("Installed default requirements successfully!"),
        "yellow",
        attrs=["bold"],
    )
    print(msg)


def create_virtual_environment(project_path):
    with console.status("[green]Creating virtual environment..."):
        subprocess.run(
            ["python", "-m", "venv", os.path.join(project_path, "venv")], check=True
        )
    msg = colored(
        (f"Created a virtual environment in {project_path}/venv"),
        "yellow",
        attrs=["bold"],
    )
    print(msg)

    with console.status("[green]Upgrading venv pip..."):
        subprocess.run(
            [f"{project_path}/venv/bin/pip", "install", "--upgrade", "pip"],
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )
    add_and_install_requirements(project_path)


def initialize_git_repository(project_path, username):
    copy_gitignore_file(project_path)
    copy_readme_file(project_path, username)
    with console.status("[green]Initializing git repo..."):
        subprocess.run(
            ["git", "init", project_path],
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )

    msg = colored(
        ("Initialized git repo successfully!"),
        "yellow",
        attrs=["bold"],
    )
    print(msg)


def initial_commit(project_path):
    with console.status("[green]Creating initial commit..."):
        subprocess.run(
            ["git", "add", "."],
            cwd=project_path,
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )
        subprocess.run(
            ["git", "commit", "-m", "initial commit"],
            cwd=project_path,
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )
    msg = colored(
        ("Created initial commit successfully!"),
        "yellow",
        attrs=["bold"],
    )
    print(msg)


def create_project_directory(project_path, interactive=False):
    try:
        os.makedirs(project_path)
        msg = colored(
            (f"Project directory '{project_path}' created."),
            "yellow",
            attrs=["bold"],
        )
        print(msg)

        return project_path

    except FileExistsError:
        msg = colored(
            (f"\nThe directory '{project_path}' already exists. Aborting."),
            "red",
            attrs=["bold"],
        )
        print(msg)
        msg = colored(
            (
                f"Delete the directory '{project_path}' and try again or give "
                "a different project name."
            ),
            "blue",
            attrs=["bold"],
        )
        print(msg)
        sys.exit(2)


def delete_path(project_path):
    if ask_user(
        colored(
            f"\nDo you want to delete the {project_path} folder to undo any changes done "
            "until now?",
            "red",
            attrs=["bold"],
        )
    ):
        shutil.rmtree(project_path)
        click.echo(
            colored(
                f"{project_path} deleted.",
                "red",
                attrs=["bold"],
            )
        )
        return True



import os
import sys

import click

from termcolor import colored

from ppieces import __version__
from ppieces.utils.cli import run_cli


@click.command()
@click.option(
    "-ni",
    "--non-interactive",
    is_flag=True,
    default=False,
    help="Run the script in non-interactive mode.",
)
@click.option(
    "-p",
    "--project-folder",
    type=click.Path(exists=True),
    required=False,
    help="The path to your projects folder.",
)
@click.option(
    "-n",
    "--project-name",
    type=str,
    required=False,
    help="The name of the new project.",
)
@click.option(
    "-v",
    "--virtual-env",
    is_flag=True,
    help="Create a virtual environment.",
)
@click.option(
    "-g",
    "--git",
    is_flag=True,
    help="Initialize a git repository (with .gitignore and README files)",
)
@click.option(
    "-pre",
    "--pre-commit",
    is_flag=True,
    help="Add pre-commit configuration.",
)
@click.option(
    "-r",
    "--ruff",
    is_flag=True,
    help="Add a ruff configuration file.",
)
@click.option(
    "-a",
    "--autoenv",
    is_flag=True,
    help="Set up autoenv.",
)
@click.option(
    "-m",
    "--makefile",
    is_flag=True,
    help="Add a default Makefile.",
)
@click.option(
    "-u",
    "--username",
    type=str,
    required=False,
    default=os.getenv("USER"),
    help="GitHub username to use in README (default: $USER)",
)
@click.option(
    "--version",
    is_flag=True,
    help="Show the version of ppieces.",
)
def main(
    non_interactive,
    project_folder,
    project_name,
    virtual_env,
    git,
    pre_commit,
    ruff,
    autoenv,
    makefile,
    username,
    version,
):
    if version:
        click.echo(__version__)
        sys.exit(1)

    if non_interactive and (not project_name or not project_folder):
        msg = colored(
            "\nBoth project name and project folder must be provided when running in "
            "non-interactive mode.\n",
            "red",
            attrs=["bold"],
        )
        print(msg)
        click.echo(main.get_help(click.Context(main)))
        sys.exit(2)

    run_cli(
        non_interactive,
        project_folder,
        project_name,
        virtual_env,
        git,
        pre_commit,
        ruff,
        autoenv,
        makefile,
        username,
    )


if __name__ == "__main__":
    main()



import os
import random

from pyfiglet import Figlet
from termcolor import colored

SCRIPTS_DIR = "scripts"
TEMPLATES_DIR = "templates"
PROJECTS_DIR = "/Users/dvitto/projects"


def welcome():
    os.system("clear")
    # isometric1, chunky, poison, cybermedium
    font = random.choice(["lean", "cyberlarge", "alligator", "larry3d", "trek"])
    fig = Figlet(font=font)
    print()
    print(colored(fig.renderText(" ppieces"), "cyan"))

    welcome_text = colored(
        ("\nWelcome to ppieces, a python project initializer!\n"),
        "red",
        attrs=["bold"],
    )
    print(welcome_text)
    print("-" * len(welcome_text))


def bye():
    msg: str = colored("\nAll set! Happy coding!\n", "blue", attrs=["bold"])
    print(msg)


def ask_user(question):
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
    prompt = " [Y/n] "
    default = "yes"
    while True:
        print(colored(f"{question}{prompt}", "cyan", attrs=["bold"]), end="")
        choice = input().lower()
        if choice == "":
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            msg = colored(
                ("Please respond with 'yes' or 'no' (or 'y' or 'n')."),
                "red",
                attrs=["bold"],
            )
            print(msg)



import os

from termcolor import colored

from ppieces.utils.commands import (
    check_precommit,
    create_virtual_environment,
    initial_commit,
    initialize_git_repository,
    install_precommit_hooks,
    setup_autoenv,
)
from ppieces.utils.copy import (
    copy_main_file,
    copy_makefile,
    copy_precommit_config,
    copy_ruff_config,
)
from ppieces.utils.validation import validate_project_name, validate_projects_folder_path


def get_project_path():
    default_projects_folder_path = os.path.join(os.getenv("HOME"), "projects")
    projects_folder_path = input(
        colored(
            "Enter the absolute path of your projects folder (default: "
            f"{default_projects_folder_path}): ",
            "cyan",
            attrs=["bold"],
        )
    )

    if not projects_folder_path:
        projects_folder_path = default_projects_folder_path

    validate_projects_folder_path(projects_folder_path)

    project_name = input(
        colored("Enter the name of your new project: ", "cyan", attrs=["bold"])
    )
    validate_project_name(project_name)

    return os.path.join(projects_folder_path, project_name)


def setup_project(
    project_path,
    options,
    username,
):
    options_mapping = {
        "virtual_env": create_virtual_environment,
        "git": initialize_git_repository,
        "autoenv": setup_autoenv,
        "ruff": copy_ruff_config,
        "pre_commit": copy_precommit_config,
        "makefile": copy_makefile,
    }

    copy_main_file(project_path)

    for option, value in options.items():
        if value:
            if option == "git":
                options_mapping[option](project_path, username)
            else:
                options_mapping[option](project_path)


def finalize_project(project_path, git, pre_commit):
    if pre_commit:
        # once we know a git repo was initialized, we can install pre-commit hooks
        if check_precommit(git):
            install_precommit_hooks(project_path)

        # otherwise, we inform the user that we can't install pre-commit hooks
        # without a git repo
        else:
            msg = colored(
                (
                    "\n\nWARNING: pre-commit is not installed. "
                    "Please install it manually."
                ),
                "red",
                attrs=["bold"],
            )
            print(msg)

    # this should be the last step since we are making the initial commit
    # AFTER all the template files are copied to the new project folder
    if git:
        initial_commit(project_path)
